(* scanner for a toy language *)

{
  open Parser
  exception Lexer_error of string
}

let digit = ['0'-'9']
let id = ['a'-'z'] ['a'-'z' '0'-'9']*

rule scan_imp = parse
  | digit+ as inum
  	{ INT (int_of_string inum) }
  | "if" { IF }
  | "else" {ELSE }
  | "while" { WHILE }
  | "skip" { SKIP }
  | ":=" { AFF }
  | "true" { TRUE }
  | "false" { FALSE }
  | "ARG" { ARG }
  | "OUT" { OUT }
  | id as text { ID text }
  | '+' { ADD }
  | '-' { SUB }
  | '*' { MULT }
  | "=" { EQ }
  | "<=" { LE }
  | "<" { LT }
  | "!" { NOT }
  | "&&" { AND }
  | "||" { OR }
  | '{' { LEFT_BRACE}
  | '}' { RIGHT_BRACE }
  | '(' { LEFT_PAR }
  | ')' { RIGHT_PAR }
  | ';' { COLON }
  | ',' { COMMA }
  | '.' { POINT }
  | "/*" [^ '\n']* "*/"	{ scan_imp lexbuf }	(* eat up one-line comments *)
  | [' ' '\t' '\n']	{ scan_imp lexbuf }	(* eat up whitespace *)
  | _ as c { raise (Lexer_error (String.make 1 c)) }
  | eof		{ EOF }
