(* Main File *)


open Type
open Lexer

(* This function generates a warning. Ocaml is that stupid 	*)
(* It splits a string into a list of strings, each element 	*)
(* being separeted by a dot in the original strings 		*)
let split_extension file = Str.split (Str.regexp "\.") file ;;

(* Given a file name, checks that the name is valid and 	*)
(* returns the same file name with the extension .ml		*)
let ml (file: string) = 
	let pieces = split_extension file in
	if List.length pieces <> 2
		then raise (Sys_error ("Invalid file name " ^ file ^ ": several or none extension") ) ;
	if List.hd (List.tl pieces) <> "imp"
		then raise (Sys_error ("Invalid file name " ^ file ^ ": wrong extension, extension .imp expected")) ;
	(List.hd pieces) ^ ".ml" ;;

(* Given a file name, opens the file for reading, opens the *)
(* .ml file for writing and call Compile.compile 			*)
let compile_file (file: string) =
	let in_chan = open_in file in
	try begin
		let out_chan = open_out (ml file) in
		try begin 
			let lexbuf = Lexing.from_channel in_chan in
			let (args,out,prog) = Parser.main scan_imp lexbuf in
			Compile.compile out_chan (List.hd (split_extension file)) args out prog ;
			close_in in_chan ;
			close_out out_chan
		end
		with
		| exc -> 
			close_out out_chan ;
			raise exc
	end
	with 
	| exc -> 
		close_in in_chan ;
		raise exc

(* Calls compile_file on every filename received in argument *)
let () =
	for i = 1 to (Array.length Sys.argv) - 1 do
		try 
			compile_file Sys.argv.(i) ; 
			let file = Sys.argv.(i) in
			Printf.printf "Compilation of %s done: output written in %s \n" file (ml file)
		with
		| Lexer_error c -> Printf.printf "Lexer error in %s: unknown character %s\n" Sys.argv.(i) c
		| Parser.Error -> Printf.printf "Parser error in %s\n" Sys.argv.(i)
		| Sys_error m -> Printf.printf "%s\n" m
	done ;;
