%{
	open Type
%}



/* Arithmetical expressions */

%token <int> INT
%token <string> ID
%token ADD SUB MULT

/* Boolean expressions */

%token TRUE
%token FALSE
%token EQ LE LT 
%token NOT AND OR

/* Programs */

%token SKIP
%token AFF
%token LEFT_BRACE RIGHT_BRACE LEFT_PAR RIGHT_PAR
%token COLON
%token IF ELSE
%token WHILE

/* Arguments declaration */

%token ARG
%token OUT
%token COMMA
%token POINT

/* End of File */
%token EOF


/* Priorities */

%left AND OR
%nonassoc NOT

%left ADD SUB        /* lowest precedence */
%left MULT         /* medium precedence */

%left COLON

%start <decl*decl*program> main

%%

main: 
| p = prog EOF
	{ ([],[],p) }
| ARG d = decl POINT p = prog EOF
	{ (d,[],p) }
| OUT d = decl POINT p = prog EOF
	{ ([],d,p) }
| ARG d_arg = decl OUT d_out = decl POINT p = prog EOF
	{ (d_arg, d_out, p) }

decl: 
| v = ID
	{ [v] }
| v = ID COMMA d = decl
	{ v::d }

prog:
| SKIP
	{ Skip }
| v = ID AFF e = a_expr
	{ Aff(v, e) }
| p1 = prog COLON p2 = prog
	{ Seq(p1,p2) }
| IF LEFT_PAR b = b_expr RIGHT_PAR LEFT_BRACE p1 = prog RIGHT_BRACE ELSE LEFT_BRACE p2 = prog RIGHT_BRACE
	{ Cond(b, p1, p2) }
| WHILE LEFT_PAR b = b_expr RIGHT_PAR LEFT_BRACE p = prog RIGHT_BRACE
	{ While(b, p) }


b_expr:
| TRUE
	{ True }
| FALSE
	{ False }
| LEFT_PAR b = b_expr RIGHT_PAR
	{ b }
| e1 = a_expr EQ e2 = a_expr
	{ Eq(e1, e2) }
| e1 = a_expr LE e2 = a_expr
	{ Le(e1, e2) }
| e1 = a_expr LT e2 = a_expr
	{ Lt(e1, e2) }
| NOT b = b_expr 
	{ Neg(b) }
| b1 = b_expr AND b2 = b_expr
	{ And(b1, b2) }
| b1 = b_expr OR b2 = b_expr
	{ Or(b1, b2) }

 
a_expr:
| i = INT
    { Nb(i) }
| v = ID
	{ Var(v) }
| LEFT_PAR e = a_expr RIGHT_PAR
    { e }
| e1 = a_expr ADD e2 = a_expr
    { Add(e1,e2) }
| e1 = a_expr SUB e2 = a_expr
    { Sub(e1,e2) }
| e1 = a_expr MULT e2 = a_expr
    { Mult(e1,e2) }




